/*
  B E A   S Y S T E M S

  Copyright (c) 2005  BEA Systems, Inc.

  All Rights Reserved. Unpublished rights reserved under the copyright laws
  of the United States. The software contained on this media is proprietary
  to and embodies the confidential technology of BEA Systems, Inc. The
  possession or receipt of this information does not convey any right to disclose
  its contents, reproduce it, or use,  or license the use, for manufacture or
  sale, the information or anything described therein. Any use, disclosure, or
  reproduction without BEA System's prior written permission is strictly prohibited.

  This sample is for example purposes only and are not designed for production use.
  It serves as a companion to the WebLogic Portal Interaction Management Guide.

  This custom event listener is an example of how you can capture
  specific events and handle them in a unique way. In this example,
  the listener persists event data in two custom database tables:
  CUSTOM_EVENTS_RESOURCE_STORE and CUSTOM_EVENTS_CONTENT_STORE.
  You would have to create these tables and their fields (shown
  below in the code) with a SQL script.

  This class requires APPLICATION\p13n_ejb.jar

  IMPORTANT NOTE: This listener is designed for simplicity, highlighting
  how to capture and respond to events. It is not production strength
  in that it writes events to the table as they occur (for rapid turnaround
  during demos) rather than using a cache (like the portal behavior
  tracking system does). Use this code as a guideline for development only.
 */

package examples.events.listeners;

import com.bea.p13n.events.EventListener;
import com.bea.p13n.events.Event;
import com.bea.p13n.util.JndiHelper;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.PreparedStatement;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Timestamp;

import javax.sql.DataSource;
import javax.naming.NamingException;
import javax.naming.Context;


public final class customEventListener
    implements EventListener
{
    // The events we're listening for and responding to. Some are WebLogic Portal's predefined events.

    private String[] eventTypes = {
        "DisplayContentEvent",
        "ClickContentEvent",
        "DisplayCampaignEvent",
        "ClickCampaignEvent",
        "ResourceDisplayedEvent",
        "ResourceDisplayedEventBT"
    };

    private final boolean debug = false;   // set to true for monitoring
    private final String  datasourceName = "weblogic.jdbc.jts.commercePool";

    private Connection connection = null;

    public customEventListener()
    {
          if( debug )
          {
              System.out.println("customEventListener: ctor()");
              System.out.flush();
          }
    }

    // Base class method that gets the types of events listened for, declared above.

    public final String[] getTypes() { return eventTypes; }


    // Base class method that lets you provide your own code for responding
    // to the event any way you like.

    public final void handleEvent( Event theEvent )
    {
          if( debug )
          {
              System.out.println("customEventListener: handleEvent("+ theEvent.getType()+")");
              System.out.flush();
          }

          String eventType  = theEvent.getType();

          if( eventType.equals("ResourceDisplayedEvent", "ResourceDisplayedEventBT") )
              processResourceDisplayedEvent( theEvent );
          else
              processContentEvents( theEvent );

          return;
    }

    // Generated for page or portlet displays, when the ResourceDisplayedEventBT event
    // is generated in the sample book.jsp or page.jsp skeleton files.  In this example,
    // event attributes are pushed to a custom table against which you can query or process
    // the data for reporting, analytics, or feeding back into your application.

    public final void processResourceDisplayedEvent ( Event theEvent )
    {
        PreparedStatement statement = null;
        try
        {
            connection = getConnection();

            connection.setAutoCommit(false);

            boolean trackedEvent = false;

                try
                {
                    Event event = theEvent;

                    String eventType  = event.getType();

                    // String pageName         = null;
                    String userId           = null;
                    String resourceSelected = null;
                    String resourceType     = null;

                    statement = connection.prepareStatement("INSERT INTO CUSTOM_EVENTS_RESOURCE_STORE  ( EVENT_DATE,  EVENT_TYPE,  RESOURCE_TYPE, RESOURCE_SELECTED, RESOURCE_ID, RESOURCE_DISPLAY_LABEL, WLS_SESSION_ID,  USER_ID  )  VALUES (?, ?, ?, ?, ?, ?, ?, ?)");

                    statement.clearParameters();
                    statement.setTimestamp(1, new Timestamp(event.getTimeStamp()));
                    statement.setString   (2, eventType);

                    resourceType = (String)event.getAttribute("resourceType");
                    if( resourceType == null || resourceType.length() == 0 )
                        resourceType = "N/A";

                    statement.setString   (3, resourceType);

                    resourceSelected = (String)event.getAttribute("resourceSelected");
                    if( resourceSelected == null || resourceSelected.length() == 0 )
                        resourceSelected = "N/A";

                    statement.setString   (4, resourceSelected);

                    String resourceId    = null;
                    String resourceLabel = null;

                    // Resources may have IDs that are different from their display labels.
                    // In 8.1, the getLabel() method of the window
                    // context object returns a unique identifier for the page or portlet
                    // (ie: portlet_3_1 or page_2), but not the actual name of the page
                    // that is returned by getTitle(). Here, "label" is used to mean
                    // "what the user sees," which is the name of the page or label.

                    resourceId = (String)event.getAttribute("resourceId");
                    if( resourceId == null || resourceId.length() == 0 )
                        resourceId = "N/A";

                    statement.setString   (5, resourceId);

                    resourceLabel = (String)event.getAttribute("resourceLabel");
                    if( resourceLabel == null || resourceLabel.length() == 0 )
                        resourceLabel = "N/A";

                    statement.setString   (6, resourceLabel);
                    statement.setString   (7, (String)event.getAttribute("session-id"));

                    userId = (String)event.getAttribute("user-id");
                    if( userId == null || userId.length() == 0 )
                        userId = "N/A";

                    statement.setString(8, userId);

                    if( debug )
                    {
                        System.out.println("- userId: "+ userId+", eventType: "+eventType);
                        System.out.println("- page or portlet name: "+ resourceLabel);
                        System.out.println("- is resource selected: "+ resourceSelected);
                        System.out.flush();
                    }

                    statement.executeUpdate();
                }
                catch(Exception e)
                {
                    System.out.println("customEventListener: got Exception during update:: "+e.getMessage());
                    System.out.flush();
                    try
                    {
                        connection.rollback();
                    }
                    catch(Exception exception1) { }
                }

            //connection.commit();
        }
        catch(NamingException ne)
        {
            // Connection would not be open either.
            // For production, one might have a fallback plan,
            // but for illustrative purposes use minimal error handling.

            System.out.println("customEventListener: got NamingException: "+ne.getMessage());
            System.out.flush();
        }
        catch(Exception e)
        {
            System.out.println("customEventListener: got Exception: "+e.getMessage());
            System.out.flush();
            try
            {
                connection.rollback();
            }
            catch(Exception exception) { }
        }
        finally
        {
            close(statement, connection);
        }
    }


    // Generated for the display and click events, which are WebLogic Portal's
    // predefined events. In this example, event attributes are pushed to a
    // custom table against which you can query or process the data for
    // reporting, analytics, or feeding back into your application.

    public final void processContentEvents ( Event theEvent )
    {
        PreparedStatement statement = null;
        try
        {
            connection = getConnection();

            connection.setAutoCommit(false);

            boolean trackedEvent = false;

                try
                {
                    Event event = theEvent;

                    // Process different kinds of events and store them to a common table.

                    // DisplayCampaignEvent occurs when campaign content, such as an ad,
                    // is displayed to the customer in a campaign placeholder. You could
                    // use this event to trigger another campaign. Analytic software uses
                    // this event to determine if a customer saw an ad as a result of a campaign.

                    // ClickCampaignEvent occurs when the user clicks on such an ad.

                    // Note that all event attributes are not persisted.

                    String eventType  = event.getType();

                    String userId     = null;
                    String campaignId = null;
                    String documentId = null;
                    String scenarioId = null;

                    statement = connection.prepareStatement("INSERT INTO CUSTOM_EVENTS_CONTENT_STORE  ( EVENT_DATE,  EVENT_TYPE,  WLS_SESSION_ID,  USER_ID,  DOCUMENT_ID, CAMPAIGN_ID, SCENARIO_ID  )  VALUES (?, ?, ?, ?, ?, ?, ?)");

                    statement.clearParameters();
                    statement.setTimestamp(1, new Timestamp(event.getTimeStamp()));
                    statement.setString   (2, eventType);
                    statement.setString   (3, (String)event.getAttribute("session-id"));

                    userId = (String)event.getAttribute("user-id");
                    if( userId == null || userId.length() == 0 )
                        userId = "N/A";

                    statement.setString(4, userId);

                    campaignId = (String)event.getAttribute("campaign-id");
                    if( campaignId == null || campaignId .length() == 0 )
                        campaignId = "N/A";

                    scenarioId = (String)event.getAttribute("scenario-id");
                    if( scenarioId == null || scenarioId .length() == 0 )
                        scenarioId = "N/A";

                    documentId = (String)event.getAttribute("document-id");
                    if( documentId == null || documentId .length() == 0 )
                        documentId = "N/A";

                    statement.setString(5, documentId);
                    statement.setString(6, campaignId);
                    statement.setString(7, scenarioId);

                    if( debug )
                    {
                        System.out.println("- userId: "+ userId+", eventType: "+eventType);
                        System.out.println("- document: "+ documentId);
                        System.out.println("- campaignId: "+ campaignId+", scenarioId: "+scenarioId);
                        System.out.flush();
                    }

                    statement.executeUpdate();
                }
                catch(Exception e)
                {
                    System.out.println("customEventListener: got Exception during update:: "+e.getMessage());
                    System.out.flush();
                    try
                    {
                        connection.rollback();
                    }
                    catch(Exception exception1) { }
                }

            //connection.commit();
        }
        catch(NamingException ne)
        {
            // Connection would not be open either.
            // For production, one might have a fallback plan,
            // but for illustrative purposes use minimal error handling.

            System.out.println("customEventListener: got NamingException: "+ne.getMessage());
            System.out.flush();
        }
        catch(Exception e)
        {
            System.out.println("customEventListener: got Exception: "+e.getMessage());
            System.out.flush();
            try
            {
                connection.rollback();
            }
            catch(Exception exception) { }
        }
        finally
        {
            close(statement, connection);
        }
    }


    // Helper methods.

    private final Connection getConnection()
        throws SQLException, NamingException
    {
        Context context = (Context)JndiHelper.getContext();
        DataSource dataSource = (javax.sql.DataSource) context.lookup( datasourceName );
        return dataSource.getConnection();
    }

    private final void close(PreparedStatement theStatement, Connection theConnection)
    {
        if(theStatement != null)
        {
            try
            {
                theStatement.close();
            }
            catch(SQLException sqlexception) { }
        }
        if(theConnection != null)
        {
            try
            {
                theConnection.close();
            }
            catch(SQLException sqlexception1) { }
        }
    }

    public boolean equals(Object theOther)
    {
        return theOther != null && theOther.getClass() == getClass();
    }
}   // end of public class customEventListener
